import typing

from .. import ConfigProvider, CredentialProvider
from .base import BaseStrategy


class SingleStrategy(BaseStrategy):
    provider_code = "single"

    def __init__(
        self, data_provider: typing.Union[typing.Type[ConfigProvider], typing.Type[CredentialProvider]]
    ) -> None:
        self.provider = data_provider

    def get(self, *args, **kwargs):
        return self.provider.get(*args, **kwargs)

    def __getattribute__(self, item):
        try:
            return super().__getattribute__(item)

        except AttributeError:
            return self.provider.__getattribute__(item)



"""
.. module:: config.providers
   :synopsis: Config Base
.. moduleauthor:: Aleksander Lavrov <github.com/egnod>
"""


import typing
from abc import ABC

from ..base import BaseProvider


class ConfigProvider(ABC, BaseProvider):
    """Base class for config providers."""


class ConfigProviderManager:
    """Manager for childeren ConfigProvider classes."""

    @staticmethod
    def get_by_code(code: str) -> typing.Optional[typing.Type[ConfigProvider]]:
        """Get config provider by provider_code.

        :param code: provider_code for search config provider
        :Example:
            .. code-block:: python

               ConfigProviderManager.get_by_code("system")

               -> SystemConfigProvider
        """
        for provider in ConfigProvider.__subclasses__():
            if provider.provider_code == code:
                return provider
        return None



"""
.. module:: __init__
   :synopsis: General Sitri class
.. moduleauthor:: Aleksander Lavrov <github.com/egnod>
"""


import typing

from loguru import logger

from .config.providers import ConfigProvider
from .credentials.providers import CredentialProvider
from .strategy.base import BaseStrategy
from .strategy.single import SingleStrategy


class Sitri:
    """Class for unite credential and config provider."""

    def __init__(
        self,
        credential_provider: typing.Optional[typing.Union[CredentialProvider, BaseStrategy]] = None,
        config_provider: typing.Optional[typing.Union[ConfigProvider, BaseStrategy]] = None,
    ):
        """

        :param credential_provider: object of credential provider
        :param config_provider: object of config provider
        """

        self.credential = None
        self.config = None

        if isinstance(credential_provider, BaseStrategy):
            self.credential = credential_provider

        elif credential_provider:
            self.credential = SingleStrategy(credential_provider)

        if isinstance(config_provider, BaseStrategy):
            self.config = config_provider

        elif config_provider:
            self.config = SingleStrategy(config_provider)

    def get_credential(self, key: str, default: typing.Any = None, **kwargs) -> typing.Union[typing.Any, None]:
        """Get value from credential provider.

        :param key: key for credential provider
        :param default: if provider return None
        """
        if not self.credential:
            logger.info("No config provider")
            return None

        variable = self.credential.get(key, **kwargs)

        return variable if variable else default

    def get_config(self, key: str, default: typing.Any = None, **kwargs) -> typing.Union[typing.Any, None]:
        """Get value from config provider.

        :param key: key for config provider
        :param default: if provider return None
        """
        if not self.config:
            logger.info("No config provider")
            return None

        variable = self.config.get(key, **kwargs)

        return variable if variable else default



import typing

from loguru import logger

from ..config.providers import ConfigProvider
from ..credentials.providers import CredentialProvider


class RedisConfigProvider(ConfigProvider):
    """Config provider for redis storage."""

    provider_code = "redis"
    _prefix = "redis"

    def __init__(self, prefix: str, redis_connector: typing.Callable):
        """

        :param prefix: prefix for create "namespace" for project variables in redis
        :param redis_connector: function return connection to Redis
        """
        self._prefix = prefix.upper()
        self._redis_get = redis_connector

    @property
    def _redis(self):
        return self._redis_get()

    def prefixize(self, key: str) -> str:
        """Get key with prefix.

        :param key: varname without prefix
        """
        return f"{self._prefix}_{key.upper()}"

    def unprefixize(self, var_name: str) -> str:
        """Remove prefix from variable name.

        :param var_name: variable name
        """

        return var_name.replace(f"{self._prefix}_", "").lower()

    @logger.catch(level="ERROR")
    def get(self, key: str, **kwargs) -> typing.Optional[str]:
        result = self._redis.get(self.prefixize(key))

        if isinstance(result, bytes):
            return result.decode()

    @logger.catch(level="ERROR")
    def keys(self) -> typing.List[str]:
        var_list = []

        for var in self._redis.keys():
            if self._prefix in var.decode():
                var_list.append(self.unprefixize(var.decode()))

        return var_list


class RedisCredentialProvider(CredentialProvider):
    """Credential provider for redis storage."""

    provider_code = "redis"
    prefix = "redis"

    def __init__(self, prefix: str, redis_connector: typing.Callable):
        """

        :param prefix: prefix for create "namespace" for project variables in redis
        :param redis_connector: function return connection to Redis
        """
        self._prefix = prefix.upper()
        self._redis_get = redis_connector

    @property
    def _redis(self):
        return self._redis_get()

    def prefixize(self, key: str) -> str:
        """Get key with prefix.

        :param key: varname without prefix
        """
        return f"{self._prefix}_{key.upper()}"

    @logger.catch(level="ERROR")
    def get(self, key: str, **kwargs) -> typing.Any:
        result = self._redis.get(self.prefixize(key))

        if isinstance(result, bytes):
            return result.decode()



from abc import ABCMeta, abstractmethod


class BaseStrategy(metaclass=ABCMeta):
    """Base class for strategies classes."""

    @property
    @abstractmethod
    def provider_code(self) -> str:
        """Code for strategy manager."""

    @abstractmethod
    def get(self, *args, **kwargs):
        """Get value by strategy."""



import contextlib
import typing

from loguru import logger

from ..config.providers import ConfigProvider
from ..credentials.providers import CredentialProvider


class VedisConfigProvider(ConfigProvider):
    """Config provider for vedis storage."""

    provider_code = "vedis"
    _hash_name = "sitri_config_hash"

    def __init__(self, vedis_connector: typing.Callable, hash_name: str = "sitri_config_hash"):
        """

        :param vedis_connector: return connection to vedis
        :param hash_name: name for hash (key-value object) in vedis
        """
        self._vedis_get = vedis_connector
        self._hash_name = hash_name

    @property
    def _vedis(self):
        return self._vedis_get()

    @property
    def _config_hash(self):
        return self._vedis.Hash(self._hash_name)

    @logger.catch(level="ERROR")
    def get(self, key: str, **kwargs) -> typing.Optional[str]:
        result = self._config_hash.get(key)

        if isinstance(result, bytes):
            return result.decode()

    @logger.catch(level="ERROR")
    def keys(self) -> typing.List[str]:
        var_list = []
        vars = self._config_hash.keys() if self._config_hash.keys() is not None else []

        for var in vars:
            var_list.append(var.decode())

        return var_list


class VedisCredentialProvider(CredentialProvider):
    """Credential provider for vedis storage."""

    provider_code = "vedis"
    prefix = "vedis"
    _hash_name = "vedis"

    def __init__(self, vedis_connector: typing.Callable, hash_name: str = "sitri_credential_hash"):
        """

        :param vedis_connector: return connection to vedis
        :param hash_name: name for hash (key-value object) in vedis
        """
        self._vedis_get = vedis_connector
        self._hash_name = hash_name

    @property
    def _vedis(self):
        return self._vedis_get()

    @property
    def _config_hash(self):
        return self._vedis.Hash(self._hash_name)

    @logger.catch(level="ERROR")
    def get(self, key: str, **kwargs) -> typing.Any:
        result = self._config_hash.get(key)

        if isinstance(result, bytes):
            with contextlib.suppress(Exception):
                return result.decode()



import os
import typing

import yaml

from ..config.providers import ConfigProvider
from ..credentials.providers import CredentialProvider

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO


class YamlConfigProvider(ConfigProvider):
    """Config provider for YAML."""

    provider_code = "yaml"

    def __init__(
        self,
        yaml_path: str = "./data.yaml",
        yaml_data: str = None,
        default_separator: str = ".",
        found_file_error: bool = True,
    ):
        """

        :param yaml_path: path to yaml file
        :param yaml_data: yaml data in string
        :param default_separator: default value separator for path-mode
        :param found_file_error: if true no file not found error raise on yaml.load
        """

        if not yaml_data:
            try:
                self._yaml = yaml.safe_load(open(os.path.abspath(yaml_path)))

            except FileNotFoundError:
                if not found_file_error:
                    self._yaml = {}

                else:
                    raise

        else:
            yaml_data = StringIO(yaml_data)
            self._yaml = yaml.safe_load(yaml_data)

        self.separator = default_separator

    def _get_by_path(self, path: str, separator: str) -> typing.Any:
        """Retrieve value from a dictionary using a list of keys.

        :param path: string with separated keys
        """
        dict_local = self._yaml.copy()
        keys = path.split(separator)

        for key in keys:
            try:
                dict_local = dict_local[int(key)] if key.isdigit() else dict_local[key]
            except Exception:
                if key not in dict_local:
                    return None

                dict_local = dict_local[key]
        return dict_local

    def _get_by_key(self, key: str) -> typing.Any:
        """Retrieve value from a dictionary using a key.

        :param key: key from json
        """

        if key in self._yaml:
            return self._yaml[key]
        else:
            return None

    def get(self, key: str, path_mode: bool = False, separator: str = None) -> typing.Optional[typing.Any]:
        """Get value from json.

        :param key: key or path for search
        :param path_mode: boolean mode switcher
        :param separator: separator for path keys in path mode
        """

        separator = separator if separator else self.separator

        if path_mode:
            return self._get_by_path(key, separator=separator)

        return self._get_by_key(key)

    def keys(self, path_mode: bool = False, separator: str = None) -> typing.List[str]:
        """Keys in json.

        :param path_mode: [future] path mode for keys list
        :param separator: [future] separators for keys in path mode
        """
        # TODO: implemented path-mode for keys list

        if not path_mode:
            return self._yaml.keys()
        else:
            raise NotImplementedError("Path-mode not implemented!")


class YamlCredentialProvider(CredentialProvider):
    """Credential provider for YAML."""

    provider_code = "yaml"

    def __init__(
        self,
        yaml_path: str = "./data.yaml",
        yaml_data: str = None,
        default_separator: str = ".",
        found_file_error: bool = True,
    ):
        """

        :param yaml_path: path to yaml file
        :param yaml_data: yaml data in string
        :param default_separator: default value separator for path-mode
        :param found_file_error: if true no file not found error raise on yaml.load
        """

        if not yaml_data:
            try:
                self._yaml = yaml.safe_load(open(os.path.abspath(yaml_path)))

            except FileNotFoundError:
                if not found_file_error:
                    self._yaml = {}

                else:
                    raise

        else:
            yaml_data = StringIO(yaml_data)
            self._yaml = yaml.safe_load(yaml_data)

        self.separator = default_separator

    def _get_by_path(self, path: str, separator: str) -> typing.Any:
        """Retrieve value from a dictionary using a list of keys.

        :param path: string with separated keys
        """
        dict_local = self._yaml.copy()
        keys = path.split(separator)

        for key in keys:
            try:
                dict_local = dict_local[int(key)] if key.isdigit() else dict_local[key]
            except Exception:
                if key not in dict_local:
                    return None

                dict_local = dict_local[key]
        return dict_local

    def _get_by_key(self, key: str) -> typing.Any:
        """Retrieve value from a dictionary using a key.

        :param key: key from yaml
        """

        if key in self._yaml:
            return self._yaml[key]
        else:
            return None

    def get(self, key: str, path_mode: bool = False, separator: str = None) -> typing.Optional[typing.Any]:
        """Get value from json.

        :param key: key or path for search
        :param path_mode: boolean mode switcher
        :param separator: separator for path keys in path mode
        """

        separator = separator if separator else self.separator

        if path_mode:
            return self._get_by_path(key, separator=separator)

        return self._get_by_key(key)

    def keys(self, path_mode: bool = False, separator: str = None) -> typing.List[str]:
        """Keys in json.

        :param path_mode: [future] path mode for keys list
        :param separator: [future] separators for keys in path mode
        """
        # TODO: implemented path-mode for keys list

        if not path_mode:
            return self._yaml.keys()
        else:
            raise NotImplementedError("Path-mode not implemented!")



import os
import typing

from ..config.providers import ConfigProvider
from ..credentials.providers import CredentialProvider


class SystemCredentialProvider(CredentialProvider):
    """Provider for get credentials from system environment."""

    _prefix: str = "system"
    provider_code: str = "system"

    def __init__(self, prefix: str):
        """
        :param prefix: prefix for create "namespace" for project variables in environment
        """
        self._prefix = prefix.upper()

    def prefixize(self, key: str) -> str:
        """Get key with prefix.

        :param key: varname without prefix
        """
        return f"{self._prefix}_{key.upper()}"

    def get(self, key: str, **kwargs) -> typing.Union[str, None]:
        """Get config variable from system env.

        :param key: key without prefix from env
        """
        return os.getenv(self.prefixize(key), None)


class SystemConfigProvider(ConfigProvider):
    """Provider for get config from system environment."""

    _prefix = "system"
    provider_code = "system"

    def __init__(self, prefix: str):
        """
        :param prefix: prefix for create "namespace" for project variables in environment
        """
        self._prefix = prefix.upper()

    def prefixize(self, key: str) -> str:
        """Get key with prefix.

        :param key: varname without prefix
        """
        return f"{self._prefix}_{key.upper()}"

    def unprefixize(self, var_name: str) -> str:
        """Remove prefix from variable name.

        :param var_name: variable name
        """

        return var_name.replace(f"{self._prefix}_", "").lower()

    def get(self, key: str, **kwargs) -> typing.Union[str, None]:
        """Get value from system env.

        :param key: key without prefix from env
        """
        return os.getenv(self.prefixize(key), None)

    def keys(self) -> typing.List[str]:
        """Get keys list with prefix from system env."""
        var_list = []

        for var in os.environ:
            if self._prefix in var:
                var_list.append(self.unprefixize(var))

        return var_list



import typing

from loguru import logger

from ..config.providers import ConfigProvider


class ConsulConfigProvider(ConfigProvider):
    """Provider for HashiCorp Consul config storage."""

    provider_code = "consul"
    folder: str

    def __init__(self, consul_connector: typing.Callable, folder: str = "sitri/") -> None:
        """

        :param consul_connector: function return connection to Consul
        :param folder: consul folder with config vars
        """
        self._consul_get = consul_connector
        self.folder = folder

    @property
    def _consul(self):
        return self._consul_get()

    @logger.catch(level="ERROR")
    def get(self, key: str, **kwargs) -> typing.Optional[typing.Any]:
        """Get value from consul by key.

        :param key: key from consul folder
        """
        index, data = self._consul.kv.get(f"{self.folder}{key}" if self.folder not in key else key)

        if data and data["Value"]:
            return data["Value"].decode()

        return None

    @logger.catch(level="ERROR")
    def keys(self) -> typing.List[typing.Any]:
        """Get keys list from consul folder."""
        index, data = self._consul.kv.get(self.folder, recurse=True)
        var_list = []

        if data:
            for var in data:
                if var["Value"]:
                    var_list.append(var["Key"])

        return var_list



import typing
from abc import ABCMeta, abstractmethod


class BaseProvider(metaclass=ABCMeta):
    """Base Provider for config and credential providers."""

    @property
    @abstractmethod
    def provider_code(self) -> str:
        """Provider code property for identity provider in manager."""

    @abstractmethod
    def get(self, key: str, **kwargs) -> typing.Optional[typing.Any]:
        """Get value from storage.

        :param key: key for get value
        :param kwargs: additional arguments for providers
        """

    def keys(self, **kwargs) -> typing.List[str]:
        """Get keys list in storage."""
        raise NotImplementedError("keys method not impl for this provider!")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['sitri',
 'sitri.config',
 'sitri.contrib',
 'sitri.credentials',
 'sitri.strategy']

package_data = \
{'': ['*']}

install_requires = \
['loguru>=0.5.1,<0.6.0', 'pytest_cases>=2.2.2,<3.0.0']

setup_kwargs = {
    'name': 'sitri',
    'version': '0.7.5',
    'description': 'Library for one endpoint config and credentials managment',
    'long_description': '\n<p align="center">\n  <a href="https://github.com/lemegetonx/sitri">\n    <img src="docs/_static/logo.gif">\n  </a>\n  <h1 align="center">\n    Sitri Configuration Library\n  </h1>\n</p>\n\n[![Build Status](https://travis-ci.org/LemegetonX/sitri.svg?branch=master)](https://travis-ci.org/LemegetonX/sitri)\n[![codecov](https://codecov.io/gh/LemegetonX/sitri/branch/master/graph/badge.svg)](https://codecov.io/gh/LemegetonX/sitri)\n![PyPI](https://img.shields.io/pypi/v/sitri)\n![Read the Docs](https://img.shields.io/readthedocs/sitri)\n[![CodeFactor](https://www.codefactor.io/repository/github/lemegetonx/sitri/badge)](https://www.codefactor.io/repository/github/lemegetonx/sitri) [![Join the chat at https://gitter.im/lemegetonx/sitri](https://badges.gitter.im/lemegetonx/sitri.svg)](https://gitter.im/lemegetonx/sitri?utm_source=badge&utm_medium=badge&utm_campaign=pr-badge&utm_content=badge)\n\nSitri - library for managing authorization and configuration data from a single object with possibly different or identical providers\n\n#  Installation\n\n```bash\npoetry add sitri\n```\n\nor\n```bash\npip3 install sitri\n```\n\n# Basics with SystemProvider\n\n```python\nfrom sitri.contrib.system import SystemCredentialProvider, SystemConfigProvider\nfrom sitri import Sitri\n\nconf = Sitri(config_provider=SystemConfigProvider(prefix="basics"),\n             credential_provider=SystemCredentialProvider(prefix="basics"))\n```\nSystem provider use system environment for get config and credential data. For unique sitri lookup to "namespace" by prefix.\n\nExample:\n\n*In console:*\n```bash\nexport BASICS_NAME=Huey\n```\n\n*In code:*\n```python\nname = conf.get_config("name")\nprint(name)  # output: Huey\n```\n\n#  Docs\nRead base API references and other part documentation on https://sitri.readthedocs.io/\n',
    'author': 'Alexander Lavrov',
    'author_email': 'egnod@ya.ru',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Egnod/sitri',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import typing

from .. import ConfigProvider, CredentialProvider
from .base import BaseStrategy


class IndexPriorityStrategy(BaseStrategy):
    """Get value from providers with priority by index in tuple."""

    provider_code = "index_priority"

    def __init__(
        self, *data_providers: typing.Union[typing.Type[ConfigProvider], typing.Type[CredentialProvider]]
    ) -> None:
        """
        :param data_providers: config or credential providers
        """

        self.providers = data_providers

    def get(self, *args, **kwargs):
        """Get value by index priority strategy.

        :param args: any args for providers
        :param kwargs: any kwargs for providers
        """
        for provider in self.providers:
            result = provider.get(*args, **kwargs)

            if result is not None:
                return result



import typing
from abc import ABC

from ..base import BaseProvider


class CredentialProvider(ABC, BaseProvider):
    """Base class for credential providers."""


class CredentialProviderManager:
    """Manager for childeren CredentialProvider classes."""

    @staticmethod
    def get_by_code(code: str) -> typing.Optional[typing.Type[CredentialProvider]]:
        """Get credential provider by provider_code.

        :param code: provider_code for search credential provider
        :Example:
            .. code-block:: python

                CredentialProviderManager.get_by_code("system")

                -> SystemCredentialProvider
        """
        for provider in CredentialProvider.__subclasses__():
            if provider.provider_code == code:
                return provider
        return None
